#!/usr/local/bin/perl
#
#
#   Read OBD pids from OBD-II compatible car/truck and log
#   them to the screen, log file, debug log file and a
#   comma delimited file.
#
#    Compatible hardware can be made using the ELM32x chip from
#      http://www.elmelectronics.com
#
#
#
#
#  Requirements!!!
#    * 386 or better processor (to run perl 5)
#    * OBD-II interface using ELM32x chip
#        -http://www.elmelectronics.com
#    * Perl5
#        -Windows http://aspn.activestate.com/ASPN/Perl/Downloads
#        -Unix    http://www.cpan.org/ports/index.html
#    * Perl Module, TERM:ReadKey
#
#
#
#


#
#Ver    Date      Auth              Notes
#1.0    20020919  Jonathans         Orig Ver



#
#  Changable Variables
#

$CONFIG = "obd.ini";










#
#  Main program
#

  require $CONFIG;


  use Term::ReadKey;




#
# Open Log & comm device
#

  &open_tty;
  &open_logs;


#
# Log date & time to show start of log file
#

  use POSIX qw(strftime);
  $val = strftime "%Y/%m/%d %H:%M:%S", localtime;
  $msg1 = "\n\n ***** Start Log  ".$val1;
  $msg2 = "  *****\n";
  $buf = $msg1.$val.$msg2;
  &print_msg(0);


#
#  Get supported PIDS
#
  $Err=0;
  while ($PIDS == 0 )
  {
    print DEV "0100\n";
    &read_com;
    if ( substr ($buf,20,1) ne ">" )
    {
      $val = "N/A"
    }
    else
    {
      $val = substr $buf,6,11;
    }
#  Debug mode
#$val = "BE 3F B8 10   >";
    $PIDS = hex($val);
    if ($PIDS == 0)
    {
      sleep 2;
      print "Trying to read supported PIDS, check HW & ignition\n";
print  substr ($buf,20,1);
      if (DEBUG) { print DEBUG_LOG_FH "Trying to read supported PIDS, check HW & ignition\n"; }
      $Err++;
      if ($Err eq 10 ) { exit 1; }
    }
  }

#
#  Decode & log supported PIDs in debug log
#

  $val = hex(substr $buf,6,2);
  &decode_pids;





#
#  Loop until it's time to quit
#

while ($key ne "q" )
{

#
# Log time
#

  $Last_Item = 0;
  $Errs = 0;
  use POSIX qw(strftime);
  $val = strftime "%H:%M:%S", localtime;
  $msg1 =  "\n\n *** ";
  $msg2 = " ***\n";
  $buf = $msg1.$val.$msg2;
  &print_msg(1);




#
#  Log any requested PID (if supported)
#

  if ($PID01 == 1 && $SPID01 == 1)
  {
      print DEV "0101\n";
      &read_com;
      if ( substr ($buf,20,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = substr $buf,6,11;
      }
      $msg1 = "Number of Error codes = ";
      $msg2 = "";
      &print_msg(2);
  }






  if ($PID03 == 1 && $SPID03 == 1 )
  {
      print DEV "0103\n";
      &read_com;
      if ( substr ($buf,14,1) ne ">" )
      {
        $Errs++;
        $val = "N/A ";
      }
      else
      {
        $val1 = substr $buf,6,2;
        if ( ($val1 & 1) == 1) {
          $val =  "Open loop"; }
        if ( ($val1 & 2) == 2) {
          $val =  "Closed loop"; }
        if ( ($val1 & 4) == 4) {
          $val =  "Open loop driving"; }
        if ( ($val1 & 8) == 8) {
          $val =  "Open loop fault"; }
        if ( ($val1 & 16) == 16) {
          $val =  "Closed loop fault"; }

      }
      $msg1 = "Fuel status           = ";
      $msg2 = "";
      &print_msg(2);
  }





  if ($PID04 == 1 && $SPID04 == 1)
  {
      print DEV "0104\n";
      &read_com;
      if (substr ($buf,11,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = sprintf ("%.2d",hex(substr $buf,6,2)*100/255);
      }
      $msg1 = "Engine Load           = ";
      $msg2 = "%";
      &print_msg(2);
  }




  if ($PID05 == 1 && $SPID05 == 1)
  {
      print DEV "0105\n";
      &read_com;
      if ( substr ($buf,11,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = hex(substr $buf,6,2)-40;
      }
      $msg1 = "Coolant temp          = ";
      $msg2 = " C";
      &print_msg(2);
  }




  if ($PID06 == 1 && $SPID06 == 1)
  {
      print DEV "0106\n";
      &read_com;
      if (substr ($buf,11,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = sprintf ("%.2d",(hex(substr $buf,6,2)-127)*.7812);
      }
      $msg1 = "Short term fuel trim  = ";
      $msg2 = "% (1)";
      &print_msg(2);
  }




  if ($PID07 == 1 && $SPID07 == 1)
  {
      print DEV "0107\n";
      &read_com;
      if (substr ($buf,11,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = sprintf ("%.2d",(hex(substr $buf,6,2)-127)*.7812);
      }
      $msg1 = "Long term fuel trim   = ";
      $msg2 = "% (1)";
      &print_msg(2);
  }



  if ($PID08 == 1 && $SPID08 == 1)
  {
      print DEV "0108\n";
      &read_com;
      if (substr ($buf,11,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = sprintf ("%.2d",(hex(substr $buf,6,2)-127)*.7812);
      }
      $msg1 = "Short term fuel trim  = ";
      $msg2 = "% (2)";
      &print_msg(2);
  }



  if ($PID09 == 1 && $SPID09 == 1)
  {
      print DEV "0109\n";
      &read_com;
      if (substr ($buf,11,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = sprintf ("%.2d",(hex(substr $buf,6,2)-127)*.7812);
      }
      $msg1 = "Long term fuel trim   = ";
      $msg2 = "% (2)";
      &print_msg(2);
  }




  if ($PID0A == 1 && $SPID0A == 1)
  {
      print DEV "010A\n";
      &read_com;
      if (substr ($buf,11,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = hex(substr $buf,6,2)*3;
      }
      $msg1 = "Intake man. Pressure  = ";
      $msg2 = " kPaG";
      &print_msg(2);
  }




  if ($PID0B == 1 && $SPID0B == 1)
  {
      print DEV "010B\n";
      &read_com;
      if (substr ($buf,11,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = hex(substr $buf,6,2);
      }
      $msg1 = "Intake man. Pressure  = ";
      $msg2 = " kPa";
      &print_msg(2);
  }



  if ($PID0C == 1 && $SPID0C == 1)
  {
      print DEV "010C\n";
      &read_com;
      $val1 = substr $buf,6,2;
      $val2 = substr $buf,9,2;
      if ( substr ($buf,14,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = int(hex($val1.$val2)/4);
      }
      $msg1 = "Engine RPM            = ";
      $msg2 = "";
      &print_msg(2);
  }



  if ($PID0D == 1 && $SPID0D == 1)
  {
      print DEV "010D\n";
      &read_com;
      if ( substr ($buf,11,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = hex(substr $buf,6,2);
      }
      $msg1 = "Vehicle speed         = ";
      $msg2 = " km/h";
      &print_msg(2);
  }




  if ($PID0E == 1 && $SPID0E == 1)
  {
      print DEV "010E\n";
      &read_com;
      if (substr ($buf,11,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = (hex(substr $buf,6,2)*.5)-64;
      }
      $msg1 = "Ignition Timing       = ";
      $msg2 = "";
      &print_msg(2);
  }




  if ($PID0F == 1 && $SPID0F == 1)
  {
      print DEV "010F\n";
      &read_com;
      if ( substr ($buf,11,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = hex(substr $buf,6,2)-40;
      }
      $msg1 = "Air intake temp       = ";
      $msg2 = " C";
      &print_msg(2);
  }





  if ($PID10 == 1 && $SPID10 == 1)
  {
      print DEV "0110\n";
      &read_com;
      $val1 = substr $buf,6,2;
      $val2 = substr $buf,9,2;
      if ( substr ($buf,14,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = sprintf ("%.1d",hex($val1.$val2)*.01);
      }
      $msg1 = "MAF Air flow         = ";
      $msg2 = " gm/sec";
      &print_msg(2);
  }





  if ($PID11 == 1 && $SPID11 == 1)
  {
      print DEV "0111\n";
      &read_com;
      if ( substr ($buf,11,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = sprintf ("%.1d",hex(substr $buf,6,2)*.3922);
      }
      $msg1 = "Trottle               = ";
      $msg2 = "%";
      &print_msg(2);
  }






  if ($PID12 == 1 && $SPID12 == 1)
  {
      print DEV "0112\n";
      &read_com;
      if ( substr ($buf,11,1) ne ">" )
      {
        $Errs++;
        $val = "N/A ";
      }
      else
      {
        $val1 = substr $buf,6,2;
        if ( ($val1 & 1) == 1) {
          $val =  "Upstream of Catalytic converter"; }
        if ( ($val1 & 2) == 2) {
          $val =  "Downstream of Catalytic converter"; }
        if ( ($val1 & 4) == 4) {
          $val =  "Atmosphere/off"; }
      }
      $msg1 = "Sec air status        = ";
      $msg2 = "";
      &print_msg(2);
  }



  if ($PID14 == 1 && $SPID14 == 1)
  {
      print DEV "0114\n";
      &read_com;
      if ( substr ($buf,14,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = hex(substr $buf,6,2)*.005;
      }
      $msg1 = "Oxy sensor V.  B1,S1  = ";
      $msg2 = "V";
      &print_msg(2);
  }

    if ($PID15 == 1 && $SPID15 == 1)
  {
      print DEV "0115\n";
      &read_com;
      if ( substr ($buf,14,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = hex(substr $buf,6,2)*.005;
      }
      $msg1 = "Oxy sensor V.  B1,S2  = ";
      $msg2 = "V";
      &print_msg(2);
  }

  if ($PID16 == 1 && $SPID16 == 1)
  {
      print DEV "0116\n";
      &read_com;
      if ( substr ($buf,14,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = hex(substr $buf,6,2)*.005;
      }
      $msg1 = "Oxy sensor V.  B1,S3  = ";
      $msg2 = "V";
      &print_msg(2);
  }

  if ($PID17 == 1 && $SPID17 == 1)
  {
      print DEV "0117\n";
      &read_com;
      if ( substr ($buf,14,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = hex(substr $buf,6,2)*.005;
      }
      $msg1 = "Oxy sensor V.  B1,S4  = ";
      $msg2 = "V";
      &print_msg(2);
  }

  if ($PID18 == 1 && $SPID18 == 1)
  {
      print DEV "0118\n";
      &read_com;
      if ( substr ($buf,14,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = hex(substr $buf,6,2)*.005;
      }
      $msg1 = "Oxy sensor V.  B2,S1  = ";
      $msg2 = "V";
      &print_msg(2);
  }

  if ($PID19 == 1 && $SPID19 == 1)
  {
      print DEV "0119\n";
      &read_com;
      if ( substr ($buf,14,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = hex(substr $buf,6,2)*.005;
      }
      $msg1 = "Oxy sensor V.  B2,S2  = ";
      $msg2 = "V";
      &print_msg(2);
  }

  if ($PID1A == 1 && $SPID1A == 1)
  {
      print DEV "011A\n";
      &read_com;
      if ( substr ($buf,14,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = hex(substr $buf,6,2)*.005;
      }
      $msg1 = "Oxy sensor V.  B2,S3  = ";
      $msg2 = "V";
      &print_msg(2);
  }

  if ($PID1B == 1 && $SPID1B == 1)
  {
      print DEV "011B\n";
      &read_com;
      if ( substr ($buf,14,1) ne ">" )
      {
        $Errs++;
        $val = "N/A "
      }
      else
      {
        $val = hex(substr $buf,6,2)*.005;
      }
      $msg1 = "Oxy sensor V.  B2,S4  = ";
      $msg2 = "V";
      &print_msg(2);
  }




#
#   Check for key press  (q = quit, or log a message)
#
  $key = ReadKey(-1);

  if ( $key ne "" )
  {
    $val = $msg{$key};
    $msg1 = "Comment               : ";
    $msg2 = "";
    $buf = $msg1.$val;
    &print_msg(2);
  }





   if ( $Errs > 5 )            #  If too many errors, re-init the com port
   {
     close DEV;
     &open_tty;

      $msg1 = "  ***  Lost Com communications *** (".$Errs.")\n";
      $msg2 = "";
      $val  = "";
      $buf  = $msg1;
      &print_msg(0);
    }
    $Errs = 0;




}


#
#   All done.   Close logs & devices
#

  close DEV;
  if ($DEBUG)  { close DEBUG_LOG_FH; }
  if ($LOG)    { close LOG_FH; }
  if ($COMMA)  { close COMMA_FH; }

exit 0;












#
#    Subroutines
#


sub open_tty                              # Open Com port and remove echo (ate0)
{
  system ("mode com1 baud=9600 parity=n data=8 stop=1 rts=off dtr=off >NUL");
  open DEV,"+>$COM_PORT" or die "Failed to open communication port\n";
  $ofh = select(DEV); $| = 1; select($ofh);
  print DEV "ate0\n";
  select(undef, undef, undef, 0.5);
  $jnk = <DEV>;

}


sub read_com                              # Read data from Com port
{
  $Timeout = 0;
  $Got_Data = 0;
  $buf = "";
  select(undef,undef,undef, 0.25);
  while ( $Timeout < 40 && $Got_Data == 0 )
  {
    select(undef,undef,undef, 0.03);
    $buf = $buf.<DEV>;
    if ($buf =~ />/ || $buf =~ /NODATA/ || $buf =~ /\?/ )
      { $Got_Data = 1; }
    $Timeout++;
  }
}



#
#    Open Logs unbuffered.   [CTRL]-C would cause data loss
#

sub open_logs
{

  if ($LOG)
  {
    open LOG_FH,">>$LOG_FILE" or die "Failed to open log file\n";
    $ofh = select(LOG_FH); $| = 1; select($ofh);
  }
  if ($DEBUG)
  {
    open DEBUG_LOG_FH,">>$DEBUG_LOG" or die "Failed to open debug log file\n";
    $ofh = select(DEBUG_LOG_FH); $| = 1; select($ofh);
  }
  if ($COMMA)
  {
    open COMMA_FH,">>$COMMA_LOG" or die "Failed to open comma delimited log file\n";
    $ofh = select(COMMA_FH); $| = 1; select($ofh);
  }
}




#
#   Print messages to log file
#
#
#  0 = Don't log to comma delimited file.
#  1 = First field of log
#  2 = All other fields
sub print_msg
{
      my $Item = shift(@_);
      if ($DEBUG)
      {
        print DEBUG_LOG_FH $buf,"\n";
      }
      if ($LOG)
      {
        print LOG_FH $msg1,$val,$msg2,"\n";
      }
      if ($SCREEN)
      {
        print $msg1,$val,$msg2,"\n";
      }
      if ($COMMA == 1  && $Item != 0)
      {
        if ($Item == 1)
          { print COMMA_FH "\n"; }
        if ($Item == 2)
          {  print COMMA_FH ","; }
        print COMMA_FH $val;
      }
}



#
#  Decode supported pids and log them.
#  Create field names in comma delimited file.
#


sub decode_pids
{
  if ( ($val & 128) == 128) { $SPID01 = 1; }
  if ( ($val & 64) == 64) { $SPID02 = 1; }
  if ( ($val & 32) == 32) { $SPID03 = 1; }
  if ( ($val & 16) == 16) { $SPID04 = 1; }
  if ( ($val & 8) == 8) { $SPID05 = 1; }
  if ( ($val & 4) == 4) { $SPID06 = 1; }
  if ( ($val & 2) == 2) { $SPID07 = 1; }
  if ( ($val & 1) == 1) { $SPID08 = 1; }

  $val = hex(substr $buf,9,2);
  if ( ($val & 128) == 128) { $SPID09 = 1; }
  if ( ($val & 64) == 64) { $SPID0A = 1; }
  if ( ($val & 32) == 32) { $SPID0B = 1; }
  if ( ($val & 16) == 16) { $SPID0C = 1; }
  if ( ($val & 8) == 8) { $SPID0D = 1; }
  if ( ($val & 4) == 4) { $SPID0E = 1; }
  if ( ($val & 2) == 2) { $SPID0F = 1; }
  if ( ($val & 1) == 1) { $SPID10 = 1; }


  $val = hex(substr $buf,12,2);
  if ( ($val & 128) == 128) { $SPID11 = 1; }
  if ( ($val & 64) == 64) { $SPID12 = 1; }
  if ( ($val & 32) == 32) { $SPID13 = 1; }
  if ( ($val & 16) == 16) { $SPID14 = 1; }
  if ( ($val & 8) == 8) { $SPID15 = 1; }
  if ( ($val & 4) == 4) { $SPID16 = 1; }
  if ( ($val & 2) == 2) { $SPID17 = 1; }
  if ( ($val & 1) == 1) { $SPID18 = 1; }

  $val = hex(substr $buf,15,2);
  if ( ($val & 128) == 128) { $SPID19 = 1; }
  if ( ($val & 64) == 64) { $SPID1A = 1; }
  if ( ($val & 32) == 32) { $SPID1B = 1; }
  if ( ($val & 16) == 16) { $SPID1C = 1; }
  if ( ($val & 8) == 8) { $SPID1D = 1; }
  if ( ($val & 4) == 4) { $SPID0E = 1; }
  if ( ($val & 2) == 2) { $SPID1F = 1; }
  if ( ($val & 1) == 1) { $SPID20 = 1; }


  if ($DEBUG)
  {
    print DEBUG_LOG_FH "PID01 = ",$SPID01,"\n";
    print DEBUG_LOG_FH "PID02 = ",$SPID02,"\n";
    print DEBUG_LOG_FH "PID03 = ",$SPID03,"\n";
    print DEBUG_LOG_FH "PID04 = ",$SPID04,"\n";
    print DEBUG_LOG_FH "PID05 = ",$SPID05,"\n";
    print DEBUG_LOG_FH "PID06 = ",$SPID06,"\n";
    print DEBUG_LOG_FH "PID07 = ",$SPID07,"\n";
    print DEBUG_LOG_FH "PID08 = ",$SPID08,"\n";
    print DEBUG_LOG_FH "PID09 = ",$SPID09,"\n";
    print DEBUG_LOG_FH "PID0A = ",$SPID0A,"\n";
    print DEBUG_LOG_FH "PID0B = ",$SPID0B,"\n";
    print DEBUG_LOG_FH "PID0C = ",$SPID0C,"\n";
    print DEBUG_LOG_FH "PID0D = ",$SPID0D,"\n";
    print DEBUG_LOG_FH "PID0E = ",$SPID0E,"\n";
    print DEBUG_LOG_FH "PID0F = ",$SPID0F,"\n";
    print DEBUG_LOG_FH "PID10 = ",$SPID10,"\n";
    print DEBUG_LOG_FH "PID11 = ",$SPID11,"\n";
    print DEBUG_LOG_FH "PID12 = ",$SPID12,"\n";
    print DEBUG_LOG_FH "PID13 = ",$SPID13,"\n";
    print DEBUG_LOG_FH "PID14 = ",$SPID14,"\n";
    print DEBUG_LOG_FH "PID15 = ",$SPID15,"\n";
    print DEBUG_LOG_FH "PID16 = ",$SPID16,"\n";
    print DEBUG_LOG_FH "PID17 = ",$SPID17,"\n";
    print DEBUG_LOG_FH "PID18 = ",$SPID18,"\n";
    print DEBUG_LOG_FH "PID19 = ",$SPID19,"\n";
    print DEBUG_LOG_FH "PID1A = ",$SPID1A,"\n";
    print DEBUG_LOG_FH "PID1B = ",$SPID1B,"\n";
    print DEBUG_LOG_FH "PID1C = ",$SPID1C,"\n";
    print DEBUG_LOG_FH "PID1D = ",$SPID1D,"\n";
    print DEBUG_LOG_FH "PID1E = ",$SPID1E,"\n";
    print DEBUG_LOG_FH "PID1F = ",$SPID2F,"\n";
    print DEBUG_LOG_FH "PID20 = ",$SPID20,"\n";
  }

  if ($COMMA)
  {
    print COMMA_FH "Time,";
    if ($PID01 == 1 && $SPID01 == 1) { print COMMA_FH "Trouble codes,"; }
    if ($PID03 == 1 && $SPID03 == 1) { print COMMA_FH "Fuel status,"; }
    if ($PID04 == 1 && $SPID04 == 1) { print COMMA_FH "Engine load %,"; }
    if ($PID05 == 1 && $SPID05 == 1) { print COMMA_FH "Coolant Temp,"; }
    if ($PID06 == 1 && $SPID06 == 1) { print COMMA_FH "S.T. fuel trim 1,"; }
    if ($PID07 == 1 && $SPID07 == 1) { print COMMA_FH "L.T. fuel trim 1,"; }
    if ($PID08 == 1 && $SPID08 == 1) { print COMMA_FH "S.T. fuel trim 2,"; }
    if ($PID09 == 1 && $SPID09 == 1) { print COMMA_FH "S.T. fuel trim 2,"; }
    if ($PID0A == 1 && $SPID0A == 1) { print COMMA_FH "Fuel press,";  }
    if ($PID0B == 1 && $SPID0B == 1) { print COMMA_FH "Intake man. press,"; }
    if ($PID0C == 1 && $SPID0C == 1) { print COMMA_FH "Engine RPM,"; }
    if ($PID0D == 1 && $SPID0D == 1) { print COMMA_FH "Speed,"; }
    if ($PID0E == 1 && $SPID0E == 1) { print COMMA_FH "Timing adv,"; }
    if ($PID0F == 1 && $SPID0F == 1) { print COMMA_FH "Intake air temp,"; }
    if ($PID10 == 1 && $SPID10 == 1) { print COMMA_FH "MAF air flow,"; }
    if ($PID11 == 1 && $SPID11 == 1) { print COMMA_FH "Throttle %,"; }
    if ($PID12 == 1 && $SPID12 == 1) { print COMMA_FH "Sec. air status,"; }
    if ($PID13 == 1 && $SPID13 == 1) { print COMMA_FH "Oxy locations,"; }
    if ($PID14 == 1 && $SPID14 == 1) { print COMMA_FH "Oxy B1-S1 V,"; }
    if ($PID15 == 1 && $SPID15 == 1) { print COMMA_FH "Oxy B1-S2 V,"; }
    if ($PID16 == 1 && $SPID16 == 1) { print COMMA_FH "Oxy B1-S3 V,"; }
    if ($PID17 == 1 && $SPID17 == 1) { print COMMA_FH "Oxy B1-S4 V,"; }
    if ($PID18 == 1 && $SPID18 == 1) { print COMMA_FH "Oxy B2-S1 V,"; }
    if ($PID19 == 1 && $SPID19 == 1) { print COMMA_FH "Oxy B2-S2 V,"; }
    if ($PID1A == 1 && $SPID1A == 1) { print COMMA_FH "Oxy B2-S3 V,"; }
    if ($PID1B == 1 && $SPID1B == 1) { print COMMA_FH "Oxy B2-S4 V,"; }
    if ($PID1C == 1 && $SPID1C == 1) { print COMMA_FH "OBD,"; }
    print COMMA_FH "Comment\n";
  }


}